@@ -47,7 +47,7 @@
 #define MINSEL 5
 
 /* istantaneous spectrum amplitude width */
-#define SPEC_DA_WIDTH 100
+#define SPEC_DA_WIDTH 150
 
 extern opt_t opt;
 extern glfer_t glfer;
@@ -328,7 +328,7 @@ static int drawing_area_configure_event(GtkWidget * widget, GdkEventConfigure *
 
   if (levbuf)
     free(levbuf);
-  D(fprintf(stderr, "levbuf[%i*%i] size requested = %i\n", pixmap_width, pixmap_height, pixmap_width * pixmap_height * sizeof(short)));
+  //fprintf(stderr, "levbuf[%i*%i] size requested = %i\n", pixmap_width, pixmap_height, pixmap_width * pixmap_height * sizeof(short));
   levbuf = calloc(pixmap_width * pixmap_height, sizeof(short));
   if (!levbuf) {
     fprintf(stderr, "Error allocating levbuf");
@@ -810,6 +810,7 @@ GtkWidget *main_window_init(int nr, float min, float max)
     freq_ad->upper = n;
     //freq_ad->pixmap_height = pixmap_height;
     gtk_drawing_area_size(GTK_DRAWING_AREA(drawing_area), pixmap_width, pixmap_height);
+    //gtk_signal_emit_by_name(GTK_OBJECT(drawing_area), "configure_event");
     /* force widget using the sdjustment to reconfigure */
     gtk_signal_emit_by_name(GTK_OBJECT(freq_ad), "changed");
 
@@ -837,7 +838,7 @@ void main_window_draw(float *psdbuf)
   float sig_pwr, floor_pwr;
   float sig_pwr_db, floor_pwr_db;
   float sig_pwr_db_new, floor_pwr_db_new;
-  float sig_level;
+  float sig_level, thr_level;
   unsigned char v;
 
   if (x >= l)
@@ -849,9 +850,14 @@ void main_window_draw(float *psdbuf)
   f_off = 0.0;
   //i_off = opt.fft_n * (fabs(opt.dfcw_dot_freq - opt.dfcw_dash_freq) / opt.sample_rate);
 
+  thr_level = opt.thr_level / 100.0; /* normalize threshold level */
+
   /* clear the spectrum amplitude pixmap */
   gdk_draw_rectangle(spec_pixmap, spec_da->style->fg_gc[GTK_WIDGET_STATE(drawing_area)], TRUE, 0, 0, SPEC_DA_WIDTH, pixmap_height);
 
+  /* this generates a linear shape spectrum, useful for testing */
+  /* for (i = 0; i < n; i++)               */
+  /*   psdbuf[i] = (255.0 * i) / (n - 1);  */
 
   /* total signal power and mean noise floor power */
   compute_floor(psdbuf, n, &sig_pwr, &floor_pwr);
@@ -897,22 +903,23 @@ void main_window_draw(float *psdbuf)
   sig_pwr_db_old = sig_pwr_db;
   floor_pwr_db_old = floor_pwr_db;
 
-//  fprintf(stderr, "sig = %.3e\tfloor = %.3e\tsig_old = %.3e\tfloor_old = %.3e\n", sig_pwr_db, floor_pwr_db, sig_pwr_db_old, floor_pwr_db_old);
-
+  //fprintf(stderr, "sig = %.3e\tfloor = %.3e\tsig_old = %.3e\tfloor_old = %.3e\n", sig_pwr_db, floor_pwr_db, sig_pwr_db_old, floor_pwr_db_old);
 
   /* i is the pixmap y coordinate */
   for (i = 0; i < n; i++) {
     /* max. value for psdbuf[i] should be 1, for sine wave input, due to */
     /* the normalization in fft.c. min. value is 0 */
     /* input relative value in dB below maximum */
+
     if (opt.scale_type == SCALE_LOG) {
       sig_level = levbuf[x * pixmap_height + i] = 10.0 * log10(psdbuf[n - i - 1]);
     } else {
       sig_level = psdbuf[n - i - 1];
       levbuf[x * pixmap_height + i] = 10.0 * log10(sig_level);
     }
-    v = f = 255 * ((sig_level - floor_pwr_db) / (sig_pwr_db - floor_pwr_db));
-    //v = f = 256 + 1.5 * levbuf[x * pixmap_height + i];
+
+    f = 255 * ((sig_level - floor_pwr_db) / (sig_pwr_db - floor_pwr_db));
+    /* be careful; v is unsigned char and will rollover when f<0 ! */
 
     /*
        f_i = psdbuf[n - i - 1];
@@ -925,13 +932,17 @@ void main_window_draw(float *psdbuf)
        }
      */
 
-    if (f < 0) {
+    if (f < 255.0 * thr_level) { /* below threshold */
       v = 0;
-      D(fprintf(stderr, "below 0!\n"));
+      D(fprintf(stderr, "below threshold (%.2f%%)!\n",  thr_level * 100.0));
     } else if (f > 255) {
       v = 255;
       D(fprintf(stderr, "over 255!\n"));
+    } else {
+      v = (f - 255.0 * thr_level) / (1.0 - thr_level);
     }
+
+
     for (j = 0; j < n_zoom; j++) {
       rgbbuf[3 * (n_zoom * i + j)] = colortab[3 * v];
       rgbbuf[3 * (n_zoom * i + j) + 1] = colortab[3 * v + 1];
@@ -939,17 +950,21 @@ void main_window_draw(float *psdbuf)
     }
 
     if (i == 0) {
-      sx_old = SPEC_DA_WIDTH * v / 255.0;
+      sx_old = SPEC_DA_WIDTH  * f / 255.0 ;
     } else {
-      sx = SPEC_DA_WIDTH * v / 255.0;
+      sx = SPEC_DA_WIDTH  * f / 255.0;
+
       /* draw spectrum amplitude pixmap */
       gdk_draw_line(spec_pixmap, spec_da->style->white_gc, sx_old, i - 1, sx, i);
       sx_old = sx;
     }
-
   }
 
+  /* draw threshold level line in current spectrum window */
+  gdk_draw_line(spec_pixmap, spec_da->style->white_gc, SPEC_DA_WIDTH * thr_level, 0, SPEC_DA_WIDTH * thr_level, n_zoom * n);
+  /* draw an entire line of the spectrogram */
   gdk_draw_rgb_image(pixmap, drawing_area->style->fg_gc[GTK_STATE_NORMAL], x, 0, 1, n_zoom * n, GDK_RGB_DITHER_NONE, rgbbuf, 3);
+  /* draw the boundary between old and new spectrogram */
   gdk_draw_line(pixmap, drawing_area->style->fg_gc[GTK_STATE_NORMAL], x + 1, 0, x + 2, n_zoom * n);
   update_rect.x = x;
   update_rect.width = 2;
