@@ -43,8 +43,8 @@ extern fft_window_t fft_windows[];
 extern int num_fft_windows;
 
 static GtkWidget *spec_window = NULL, *qrss_window = NULL, *tx_test_window = NULL, *port_window = NULL;
-static GtkWidget *sn_bu, *qrss_bu, *dfcw_bu, *serial_bu, *parallel_bu, *hendrix_bu;
-static GtkWidget *offset_en, *fft_n_en, *fft_overlap_en, *fft_a_en, *mtm_n_en, *mtm_w_en, *mtm_k_en, *mtm_overlap_en, *hparma_n_en, *hparma_t_en, *hparma_p_e_en, *sample_rate_en, *dot_time_en, *lin_scale_bu, *log_scale_bu, *ptt_delay_en, *sidetone_fr_en, *dfcw_dot_fr_en, *dfcw_dash_fr_en;
+static GtkWidget *sn_bu, *be_keyup_bu, *be_keydown_bu, *qrss_bu, *dfcw_bu, *serial_bu, *parallel_bu, *hendrix_bu;
+static GtkWidget *offset_en, *fft_n_en, *fft_overlap_en, *fft_a_en, *mtm_n_en, *mtm_w_en, *mtm_k_en, *mtm_overlap_en, *hparma_n_en, *hparma_t_en, *hparma_p_e_en, *sample_rate_en, *dot_time_en, *wpm_en, *lin_scale_bu, *log_scale_bu, *ptt_delay_en, *sidetone_fr_en, *dfcw_dot_fr_en, *dfcw_dash_fr_en, *be_pause_en, *level_en;
 static GtkWidget *serial_dev_cb, *parallel_dev_cb;
 
 
@@ -69,6 +69,8 @@ static palette_t palettes[] = {
 
 static gint num_palettes = sizeof(palettes) / sizeof(palettes[0]);
 
+static void dot_time_en_changed(GtkWidget * widget, gpointer * data);
+static void wpm_en_changed(GtkWidget * widget, gpointer * data);
 
 /* save_event, called by the save button, saves the preferences to file     
  */
@@ -197,6 +199,10 @@ static void spec_prefs_to_widgets(void)
   gtk_entry_set_text(GTK_ENTRY(offset_en), tmp_str);
   g_free(tmp_str);
 
+  tmp_str = g_strdup_printf("%.1f", opt.thr_level);
+  gtk_entry_set_text(GTK_ENTRY(level_en), tmp_str);
+  g_free(tmp_str);
+
   /* FFT */
   tmp_str = g_strdup_printf("%i", opt.fft_n);
   gtk_entry_set_text(GTK_ENTRY(fft_n_en), tmp_str);
@@ -271,6 +277,11 @@ static void spec_widgets_to_prefs(GtkWidget * widget, gpointer data)
   opt.offset_freq = atof(tmp_str);
   g_free(tmp_str);
 
+  tmp_str = gtk_editable_get_chars(GTK_EDITABLE(level_en), 0, -1);
+  opt.thr_level = atof(tmp_str);
+  g_free(tmp_str);
+
+
   /* FFT */
   tmp_str = gtk_editable_get_chars(GTK_EDITABLE(fft_n_en), 0, -1);
   opt.fft_n = atoi(tmp_str);
@@ -511,6 +522,9 @@ void spec_settings_dialog(GtkWidget * widget, gpointer data)
   /* offset entry */
   compound_entry("Offset [Hz] :", &offset_en, tmp_vb);
   gtk_tooltips_set_tip(glfer.tt, offset_en, "Spectrogram frequency offset", NULL);
+  /* threshold level entry */
+  compound_entry("Threshold level [%] :", &level_en, tmp_vb);
+  gtk_tooltips_set_tip(glfer.tt, level_en, "Display threshold level [0-99.9]", NULL);
   /* scale type frame */
   ty_fr = gtk_frame_new("Scale");
   gtk_box_pack_start(GTK_BOX(tmp_vb), ty_fr, FALSE, FALSE, 10);
@@ -635,6 +649,10 @@ static void qrss_prefs_to_widgets(void)
   gtk_entry_set_text(GTK_ENTRY(dfcw_dash_fr_en), tmp_str);
   g_free(tmp_str);
 
+  tmp_str = g_strdup_printf("%.1f", opt.beacon_pause);
+  gtk_entry_set_text(GTK_ENTRY(be_pause_en), tmp_str);
+  g_free(tmp_str);
+  
   if (opt.tx_mode == QRSS)
     gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(qrss_bu), TRUE);
   else if (opt.tx_mode == DFCW)
@@ -643,6 +661,15 @@ static void qrss_prefs_to_widgets(void)
     g_print("opt.tx_mode = %i\n", opt.tx_mode);
     g_assert_not_reached();
   }
+
+  if (opt.beacon_tx_pause == FALSE)
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(be_keyup_bu), TRUE);
+  else if (opt.beacon_tx_pause == TRUE)
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(be_keydown_bu), TRUE);
+  else {
+    g_print("opt.beacon_tx_pause = %i\n", opt.beacon_tx_pause);
+    g_assert_not_reached();
+  }
 }
 
 
@@ -670,6 +697,17 @@ static void qrss_widgets_to_prefs(GtkWidget * widget, gpointer data)
   opt.dfcw_dash_freq = atof(tmp_str);
   g_free(tmp_str);
 
+  tmp_str = gtk_editable_get_chars(GTK_EDITABLE(be_pause_en), 0, -1);
+  opt.beacon_pause = atof(tmp_str);
+  g_free(tmp_str);
+  
+  if (GTK_TOGGLE_BUTTON(be_keyup_bu)->active)
+    opt.beacon_tx_pause = FALSE;
+  else if (GTK_TOGGLE_BUTTON(be_keydown_bu)->active)
+    opt.beacon_tx_pause = TRUE;
+  else
+    g_assert_not_reached();
+
   if (GTK_TOGGLE_BUTTON(qrss_bu)->active)
     opt.tx_mode = QRSS;
   else if (GTK_TOGGLE_BUTTON(dfcw_bu)->active)
@@ -682,12 +720,47 @@ static void qrss_widgets_to_prefs(GtkWidget * widget, gpointer data)
 }
 
 
+static void wpm_en_changed(GtkWidget * widget, gpointer * data)
+{
+  gchar *tmp_str;
+  float tmp_f;
+
+  tmp_str = gtk_editable_get_chars(GTK_EDITABLE(wpm_en), 0, -1);
+  tmp_f = atof(tmp_str);
+  g_free(tmp_str);
+  opt.dot_time = 1000.0 * 60.0 / (tmp_f * 50);
+  tmp_str = g_strdup_printf("%.1f", opt.dot_time);
+  gtk_signal_handler_block_by_func(GTK_OBJECT(dot_time_en), GTK_SIGNAL_FUNC(dot_time_en_changed), data);
+  gtk_entry_set_text(GTK_ENTRY(dot_time_en), tmp_str);
+  gtk_signal_handler_unblock_by_func(GTK_OBJECT(dot_time_en), GTK_SIGNAL_FUNC(dot_time_en_changed), data);
+  g_free(tmp_str);
+}
+
+
+static void dot_time_en_changed(GtkWidget * widget, gpointer * data)
+{
+  gchar *tmp_str;
+  float tmp_f1, tmp_f2;
+
+  tmp_str = gtk_editable_get_chars(GTK_EDITABLE(dot_time_en), 0, -1);
+  tmp_f1 = atof(tmp_str);
+  g_free(tmp_str);
+  tmp_f2 = 60.0 / (tmp_f1 * 50 / 1000.0);
+  tmp_str = g_strdup_printf("%.1f", tmp_f2);
+  gtk_signal_handler_block_by_func(GTK_OBJECT(wpm_en), GTK_SIGNAL_FUNC(wpm_en_changed), data);
+  gtk_entry_set_text(GTK_ENTRY(wpm_en), tmp_str);
+  gtk_signal_handler_unblock_by_func(GTK_OBJECT(wpm_en), GTK_SIGNAL_FUNC(wpm_en_changed), data);
+  //  gtk_signal_emit_stop_by_name(GTK_OBJECT(wpm_en), "changed");
+  g_free(tmp_str);
+}
+
+
 void qrss_settings_dialog(GtkWidget * widget, gpointer data)
 {
   GtkWidget *temp_bu;
   GtkWidget *tmp_vb, *tmp_hb;
-  GtkWidget *ti_fr, *si_fr, *mo_fr;
-  GSList *mode_gr;
+  GtkWidget *ti_fr, *be_fr, *si_fr, *mo_fr;
+  GSList *mode_gr, *be_gr;
 
   if (qrss_window && qrss_window->window) {
     gtk_widget_map(qrss_window);
@@ -705,8 +778,8 @@ void qrss_settings_dialog(GtkWidget * widget, gpointer data)
 
   gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(qrss_window)->vbox), 5);
 
-  /* Timing frame */
-  ti_fr = gtk_frame_new("Timing");
+  /* QRSS/DFCW timings frame */
+  ti_fr = gtk_frame_new("QRSS/DFCW timings");
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(qrss_window)->vbox), ti_fr, FALSE, FALSE, 10);
   gtk_widget_show(ti_fr);
   /* Timing vbox */
@@ -716,9 +789,36 @@ void qrss_settings_dialog(GtkWidget * widget, gpointer data)
   /* Timing entries */
   compound_entry("Dot length [ms] :", &dot_time_en, tmp_vb);
   gtk_tooltips_set_tip(glfer.tt, dot_time_en, "dot time duration", NULL);
+  compound_entry("Words per minute [WPM] :", &wpm_en, tmp_vb);
+  gtk_tooltips_set_tip(glfer.tt, wpm_en, "keying speed in word per minute", NULL);
+  gtk_signal_connect(GTK_OBJECT(dot_time_en), "changed", GTK_SIGNAL_FUNC(dot_time_en_changed), NULL);
+  gtk_signal_connect(GTK_OBJECT(wpm_en), "changed", GTK_SIGNAL_FUNC(wpm_en_changed), NULL);
   compound_entry("PTT delay [ms] :", &ptt_delay_en, tmp_vb);
   gtk_tooltips_set_tip(glfer.tt, ptt_delay_en, "delay between PTT and KEY", NULL);
 
+  /* Beacon mode timings frame */
+  be_fr = gtk_frame_new("Beacon mode timings");
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(qrss_window)->vbox), be_fr, FALSE, FALSE, 10);
+  gtk_widget_show(be_fr);
+  /* Timing vbox */
+  tmp_vb = gtk_vbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(be_fr), tmp_vb);
+  gtk_widget_show(tmp_vb);
+  /* Timing entries */
+  compound_entry("Pause between messages [s] :", &be_pause_en, tmp_vb);
+  gtk_tooltips_set_tip(glfer.tt, be_pause_en, "Pause between beacon transmission", NULL);
+  /* beacon pause with key up or down radio buttons */
+  be_keyup_bu = gtk_radio_button_new_with_label(NULL, "Key up between pauses");
+  gtk_tooltips_set_tip(glfer.tt, be_keyup_bu, "Transmitter OFF between pauses", NULL);
+  be_gr = gtk_radio_button_group(GTK_RADIO_BUTTON(be_keyup_bu));
+  be_keydown_bu = gtk_radio_button_new_with_label(be_gr, "Key down between pauses");
+  gtk_tooltips_set_tip(glfer.tt, be_keydown_bu, "Transmitter ON between pauses", NULL);
+  be_gr = gtk_radio_button_group(GTK_RADIO_BUTTON(be_keydown_bu));
+  gtk_box_pack_start(GTK_BOX(tmp_vb), be_keyup_bu, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(tmp_vb), be_keydown_bu, TRUE, TRUE, 0);
+  gtk_widget_show(be_keyup_bu);
+  gtk_widget_show(be_keydown_bu);
+
   /* Sidetone frame */
   si_fr = gtk_frame_new("Sidetone");
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(qrss_window)->vbox), si_fr, FALSE, FALSE, 10);
