@@ -78,6 +78,8 @@ extern opt_t opt;
 static int ksound_fd = -1;	/* console file descriptor */
 static int serial_fd = -1;	/* serial port file descriptor */
 static int ksound_opened = FALSE;
+static int repeat_msg = FALSE; /* repeat message transmission (beacon mode) */
+
 
 /* function prototypes */
 static void send_next_element(int signal);
@@ -193,6 +195,7 @@ static void emit_sound(float freq)
   }
 }
 
+
 int open_parport(char *device)
 {
 #if HAVE_SYS_IO_H
@@ -219,6 +222,7 @@ int open_parport(char *device)
   return 0;
 }
 
+
 /* open_serial_port() - open serial port
  *
  * returns the file descriptor on success or -1 on error
@@ -390,6 +394,7 @@ void key_down(void)
   D(printf("KEY DOWN\n"));	/* for debug */
 }
 
+
 void key_up(void)
 {
   int status;
@@ -481,12 +486,23 @@ static void qrss_dot_pause(int trig_signal)
     set_alarm_timer(5.0 * opt.dot_time);
   } else if (str_index < str_len) {
     /* end of the string, end of transmission */
-    str_index++;		/* this will signal that last character was sent */
-    signal(SIGALRM, qrss_dot_pause);
-    set_alarm_timer(opt.ptt_delay);
-  } else {
-    /* string has ended */
-    ptt_off();
+    if (repeat_msg == FALSE) { /* not in beacon mode */
+      str_index++; /* signal that last character was sent */
+      signal(SIGALRM, qrss_dot_pause);
+      set_alarm_timer(opt.ptt_delay);
+    } else { /* in beacon mode */
+      if (opt.beacon_tx_pause == TRUE) {
+	key_down();
+	if (opt.sidetone == TRUE)
+	  emit_sound(opt.sidetone_freq);
+      }
+      str_index = 0; /* reset pointer to beginning of string */
+      signal(SIGALRM, send_next_char);
+      /* start transmissions after  */
+      set_alarm_timer(opt.beacon_pause * 1000); /* pause is in seconds */
+    }
+  } else { /* not in beacon mode */
+    ptt_off(); /* transmission ends here */
   }
 }
 
@@ -662,6 +678,8 @@ void send_string(char *str)
   str_index = 0;
   str_len = strlen(s);
 
+  repeat_msg = opt.beacon_mode;
+
   switch (opt.tx_mode) {
   case QRSS:
     /* close PTT */
@@ -685,6 +703,8 @@ void send_string(char *str)
 
 void stop_tx(void)
 {
+  /* avoid restart of message transmission */
+  repeat_msg = FALSE;
   /* stop sidetone */
   emit_sound(0);
   /* hack, pretend we are at the end of the string */
@@ -714,6 +734,9 @@ float string_duration(char *str)
   char c;
   float overall_time = 0.0;
 
+  /* add the PTT delay at start and end of transmissions */
+  overall_time += 2.0 * opt.ptt_delay;
+
   for (i = 0; i < strlen(str); i++) {
     /* ensure uppercase */
     c = toupper(str[i]);
@@ -782,6 +805,7 @@ float string_duration(char *str)
   return overall_time;
 }
 
+
 int open_ksound(void)
 {
   /* To be used as the ksound_fd in ioctl(). */
@@ -792,6 +816,7 @@ int open_ksound(void)
     return (ksound_opened = TRUE);
 }
 
+
 void close_ksound(void)
 {
   if (ksound_opened == TRUE)
